package Background;
#
# phroggy.com 5.06
#
# (C) 2004 Andy Lyttle, all rights reserved.
# This code may be distributed and modified under the terms of
# the GNU General Public License.
# http://phroggy.com/opensource/
# 

use strict;
use warnings;
use Paths;
use PhroggyIO;
use Theme;
use Localize;

sub new {
  # Arguments: background name

  # Reads the specified background from the backgrounds directory, if it exists.

  my($pkg,$bg)=@_;
  my %data=('exists'=>0,'name'=>$bg,'title'=>{});
  if($bg=~/[^a-z0-9\-]/) {
    Abort('Tried to read a background containing invalid characters.');
  }
  my $info=ReadFile("$Paths::File{'bg'}/$bg.info",undef,'notrans');
  if($info) {
    $data{'exists'}=1;
    foreach my $line (split /\n/,$info) {
      $data{'title'}->{$1}=$2 if($line && $line=~/^(.*?)=(.*)$/);
    }
  }
  return bless \%data, $pkg;
}

sub Valid {
  # Returns: 1 if this background is valid, 0 if not

  my($self)=@_;
  return $self->{'exists'};
}

sub Name {
  # Returns: name of this background
  
  my($self)=@_;
  return $self->{'name'};
}

sub Title {
  # Returns: title of this background (localized)
  
  my($self)=@_;
  return $self->{'title'}->{ChooseLanguage(keys %{$self->{'title'}})};
}

sub OKForTheme {
  # Arguments: theme object
  # Returns: 1 if OK, 0 if not
  
  # If the theme has exclusivebg set, and this background isn't the
  # exclusive background allowed for the theme, then this background
  # is not OK for the theme

  my($self,$theme)=@_;
  my $ok=$self->Valid();
  $ok=0 if($ok && $theme->Info('exclusivebg') && $self->{'name'} ne $theme->Info('exclusivebg'));
  return $ok;
}

sub GetTitleList {
  my($self)=@_;
  return %{$self->{'title'}};
}

1;
