package Counter;
#
# phroggy.com 5.3
#
# (C) 2004 Andy Lyttle, all rights reserved.
# This code may be distributed and modified under the terms of
# the GNU General Public License.
# http://phroggy.com/opensource/
# 
# These are functions for incrementing the counter, and logging hits.
#

BEGIN {
  use base 'Exporter';
  our @EXPORT=qw(
    SetLock
    ClearLock
    GetCounter
    LogHit
  );
}

use strict;
use warnings;
use Paths;
use Browser;
use PhroggyIO;
use Theme;
use Localize;

our($Lock,$Incremented,$Robot);

sub SetLock {
  $Lock=0;
  for(my $x=0; $x<2; $x++) {
    if(open LOCK,">>$Paths::File{'temp'}/lockfile.txt") {
      print LOCK "$$\n";
      close LOCK;
      if(open LOCK,"$Paths::File{'temp'}/lockfile.txt") {
        my $pid=<LOCK>;
        close LOCK;
        if($pid && $pid==$$) {
          # our lockfile was created successfully
          $Lock=1;
        }
      }
    }
    last if($Lock); # no need to keep looping once we've got it
    # lockfile is owned by someone else; wait for them to finish
    sleep 1 if($x==0);
  }
  Alert(Translate('Lockfile stuck')) unless($Lock);
}

sub ClearLock {
  unlink("$Paths::File{'temp'}/lockfile.txt") if($Lock);
}

sub GetCounter {
  # Increment the counter, unless:
  # the quote cookie is set
  # the user_agent is a robot
  # or this IP has visited within the last hour

  # New count is stored in $main::Count
  # $Incremented is 1 if incremented, 0 if not

  $main::Count=ReadFile("$Paths::File{'data'}/count.data",'nocache','notrans');
  $main::Count=0 unless($main::Count && $main::Count=~/^\d+$/);
  chomp($main::Count);
  $Incremented=1;

  if($Lock) { # don't write if we didn't successfully create a lockfile
    my %IP=();
    if(open(LIST,"$Paths::File{'data'}/IPs.data")) {
      while(<LIST>) {
        if(/^(\S+)\s+(\S+)$/) {
          # forget anything older than an hour
          $IP{$1}=$2 if($2>(time()-3600));
        }
      }
      close(LIST);
    }
    $Incremented=0 if($IP{$ENV{'REMOTE_ADDR'}});
    $IP{$ENV{'REMOTE_ADDR'}}=time;
    if(open(LIST,">$Paths::File{'data'}/IPs.data")) {
      my @temp=%IP;
      while(@temp) {
        print LIST shift(@temp)."\t".shift(@temp)."\n";
      }
      close LIST;
    } else {
      Alert('Unable to write to IPs.data.');
    }
  } else {
    # lockfile is stuck
    $Incremented=0;
  }

  # don't count visitors with our cookie, and don't count robots
  my $b=$ENV{'HTTP_USER_AGENT'}?$ENV{'HTTP_USER_AGENT'}:'';
  $Robot=1 if($b eq ''
      || $b=~/Googlebot/
      || $b=~/ia_archiver/
      || $b=~/Openbot\//
      || $b=~/Slurp/
      || $b=~/^Wget\//  # not a bot but not a browser either
      || $b=~/^cosmos\//
      || $b=~/^marvin\/infoseek/
      || $b=~/crawler/ # will match several different bots
      || $b=~/^psbot\//
      || $b=~/Web Server Survey/
      || $b=~/^InfoNaviRobot/
      || $b=~/grub-client/
      || $b=~/compatible; IQ/
      || $b=~/BecomeBot\//
      || $b=~/msnbot\//
  );
  $Incremented=0 if($Browser::Cookies{'quote'} || $Robot);

  if($Incremented) {
    $main::Count++;
    if(open COUNT,">$Paths::File{'data'}/count.data") {
      print COUNT "$main::Count\n";
      close COUNT;
    } else {
      Alert('Unable to write to count.data.');
    }
  }
}

sub LogHit {
  # Arguments: filename
  
  my($filename)=@_;
  my($sec,$min,$hour,$mday,$mon,$year)=localtime;
  my $date=sprintf '%02u/%02u/%02u %02u:%02u:%02u',$mon+1,$mday,substr($year+1900,-2,2),$hour,$min,$sec;
  no warnings 'uninitialized';
  open LOG,">>$Paths::File{'data'}/web.log";
  print LOG "$date ($filename, count=$main::Count".($Incremented?'':'*');
  print LOG ", theme=".$main::Theme->Name.", background=".$main::Background->Name.")\n";
  print LOG "    by $ENV{'REMOTE_ADDR'} ($ENV{'REMOTE_HOST'})\n";
  print LOG "    with".($Robot?'*':'')." $ENV{'HTTP_USER_AGENT'} ($Browser::BrowserLevel)\n";
  print LOG "    from $ENV{'HTTP_REFERER'}\n";
  close LOG;
}


1;
