package Localize;
#
# phroggy.com 5.4
#
# (C) 2009 Andy Lyttle, all rights reserved.
# This code may be distributed and modified under the terms of
# the GNU General Public License.
# http://phroggy.com/opensource/
# 
# These functions are used to localize text into non-English languages,
# based on the user's language preferences.
#

BEGIN {
  use base 'Exporter';
  our @EXPORT=qw(
    ChooseLanguage
    Translate
    ShortDate
    LongDate
    SearchMatches
    GetLanguages
    ThemeError
    LocalizeFilename
    GenerateCaptcha
    NumToString
  );
}

use strict;
use warnings;
use PhroggyIO;
use Browser;
use Paths;

our(%Matrix,%Matches,%ThemeError,%Languages,@Primary,@Secondary,@Quality,$GotLanguages);


my $L='en'; ### English ###

$Matrix{'Navigation'}{$L}='Navigation';
$Matrix{'Themes'}{$L}='Themes';
$Matrix{'Theme'}{$L}='Theme';
$Matrix{'Background'}{$L}='Background';
$Matrix{'Change'}{$L}='Change';
$Matrix{'Random Quote'}{$L}='Random Quote';
$Matrix{'Welcome to'}{$L}='Welcome to';
$Matrix{'Counter'}{$L}='You are visitor #';

$Matrix{'Weblog Index'}{$L}='Weblog Index';
$Matrix{'Weblog Navigation'}{$L}='Weblog Navigation';
$Matrix{'Weblog entry'}{$L}='Weblog entry #';
$Matrix{'Search'}{$L}='Search';
$Matrix{'Untitled Page'}{$L}='Untitled Page';
$Matrix{'Search Results'}{$L}='Search Results';
$Matrix{'plus'}{$L}='plus';
$Matrix{'minus'}{$L}='minus';
$Matrix{'Lockfile stuck'}{$L}='A lockfile is stuck; please send me e-mail to let me know.';
$Matrix{'Language ignored'}{$L}='The specified language contains invalid characters and will be ignored.';
$Matches{$L}=sub {
  my($num)=@_;
  return ('(No matches)','(1 match)',"($num matches)")[$num<2?$num:2];
};
$ThemeError{$L}=sub {
  my($thErr,$bgErr,$thUse,$bgUse)=@_;
  my %strings=();
  $strings{'Theme missing'}='The specified theme is invalid (it does not exist or could not be read)';
  $strings{'Theme error'}='The specified theme is invalid (something\'s wrong with it)';
  $strings{'Theme bad layout'}='The specified theme is invalid (it does not specify a valid layout)';
  $strings{'Theme bad background'}='The specified theme requires a background that is invalid';
  $strings{'Default background invalid'}='The default background for the specified theme is invalid';
  $strings{'Exclusive background'}='The specified background is not valid for this theme';
  $strings{'Bad background'}='The specified background is invalid';
  
  Abort("Bad theme error: $thErr.") if($thErr && index($thErr,join '@',keys %strings)!=-1);
  Abort("Bad bg error: $bgErr.") if($bgErr && index($bgErr,join '@',keys %strings)!=-1);
  if($thErr && $bgErr) {
    my $temp=$strings{$bgErr};
    substr($temp,0,1)=lc(substr($temp,0,1)); # downcase first letter
    return $strings{$thErr}." and $temp, so instead we're using the $thUse theme and the $bgUse background.";
  } elsif($thErr) {
    return $strings{$thErr}.", so we're using the $thUse theme instead.";
  } elsif($bgErr) {
    return $strings{$bgErr}.", so we're using the $bgUse background instead.";
  }
};


$L='es'; ### Spanish ###

$Matrix{'Navigation'}{$L}='Navigaci&oacute;n';
$Matrix{'Themes'}{$L}='Temas';
$Matrix{'Theme'}{$L}='Tema';
$Matrix{'Background'}{$L}='Fondo';
$Matrix{'Change'}{$L}='Cambiar';
$Matrix{'Random Quote'}{$L}='Cotizaci&oacute;n al Azar';
$Matrix{'Welcome to'}{$L}='Bi&eacute;nvenidos a';
$Matrix{'Counter'}{$L}='Ud. es visitante no. ';

$Matrix{'Weblog Index'}{$L}='&Iacute;ndice Weblog';
$Matrix{'Weblog Navigation'}{$L}='Navigaci&oacute;n Weblog';
$Matrix{'Weblog entry'}{$L}='Apuntamiento weblog no. ';
$Matrix{'Search'}{$L}='Buscar';
$Matrix{'Untitled Page'}{$L}='P&aacutegina Sin T&iacute;tulo';
$Matrix{'Search Results'}{$L}='Resultados de Buscar';
$Matrix{'plus'}{$L}='y';
$Matrix{'minus'}{$L}='menos';
$Matrix{'Lockfile stuck'}{$L}='Se pega un lockfile; por favor env&iacute;eme e-mail para decirme.';
$Matrix{'Language ignored'}{$L}='Esta lengua tiene car&aacute;cteres inv&aacute;lidos y ser&aacute; ignorada.';
$Matches{$L}=sub {
  my($num)=@_;
  return ('(No hay resultados)','(1 resultado)',"($num resultados)")[$num<2?$num:2];
};
$ThemeError{$L}=sub {
  my($thErr,$bgErr,$thUse,$bgUse)=@_;
  my %strings=();
  $strings{'Theme missing'}="El tema especificado es inv&aacute;lido (no lo existe o no lo se puede leer)";
  $strings{'Theme error'}="El tema especificado es inv&aacute;lido (hay algo mal con el tema)";
  $strings{'Theme bad layout'}="El tema especificado es inv&aacute;lido (no especifica una disposici&oacute;n v&aacute;lida)";
  $strings{'Theme bad background'}="El tema especificado requiere un fondo que sea inv&aacute;lido";
  $strings{'Default background invalid'}="El fondo defecto del tema es inv&aacute;lido";
  $strings{'Exclusive background'}="El fondo especificado no es v&aacute;lido para esta tema";
  $strings{'Bad background'}="El fondo especificado es inv&aacute;lido";
  
  Abort("Bad theme error: $thErr.") if($thErr && index($thErr,join '@',keys %strings)!=-1);
  Abort("Bad bg error: $bgErr.") if($bgErr && index($bgErr,join '@',keys %strings)!=-1);
  if($thErr && $bgErr) {
    my $temp=$strings{$bgErr};
    substr($temp,0,1)=lc(substr($temp,0,1)); # downcase first letter
    return $strings{$thErr}." y $temp, as&iacute; que en lugar usamos el tema $thUse y el fondo $bgUse.";
  } elsif($thErr) {
    return $strings{$thErr}.", as&iacute; que usamos el tema $thUse en lugar.";
  } elsif($bgErr) {
    return $strings{$bgErr}.", as&iacute; que usamos el fondo $bgUse en lugar.";
  }
};


$L='de'; ### German ###

$Matrix{'Navigation'}{$L}='Navigation';
$Matrix{'Themes'}{$L}='Themen';
$Matrix{'Theme'}{$L}='Thema';
$Matrix{'Background'}{$L}='Hintergrund';
$Matrix{'Change'}{$L}='Wechsle';
$Matrix{'Random Quote'}{$L}='Zuf&auml;llige Anf&uuml;hrung';
$Matrix{'Welcome to'}{$L}='Willkommen zu';
$Matrix{'Counter'}{$L}='Sie sind Besucher Nr. ';

$Matrix{'Weblog Index'}{$L}='Weblog Index';
$Matrix{'Weblog Navigation'}{$L}='Weblog Navigation';
$Matrix{'Weblog entry'}{$L}='Weblog Eintrag Nr. ';
$Matrix{'Search'}{$L}='Such';
$Matrix{'Untitled Page'}{$L}='Keine Seite &Uuml;berschrift';
$Matrix{'Search Results'}{$L}='Such Ergebnisse';
$Matrix{'plus'}{$L}='plus';
$Matrix{'minus'}{$L}='minus';
$Matrix{'Lockfile stuck'}{$L}='Einen Lockfile steckt; sende mich bitte eine E-Mail so ich kann wissen.';
$Matrix{'Language ignored'}{$L}='Diese spezifizierte Sprache hat kr&auml;nkliche Charakters und wird ignoriert sein.';
$Matches{$L}=sub {
  my($num)=@_;
  return ('(Keine Gleiche)',"($num Gleiche)")[$num?1:0];
};
$ThemeError{xxxde}=sub {
  my($thErr,$bgErr,$thUse,$bgUse)=@_;
  my %strings=();
  $strings{'Theme missing'}='The specified theme is invalid (it does not exist or could not be read)';
  $strings{'Theme error'}='The specified theme is invalid (something\'s wrong with it)';
  $strings{'Theme bad layout'}='The specified theme is invalid (it does not specify a valid layout)';
  $strings{'Theme bad background'}='The specified theme requires a background that is invalid';
  $strings{'Default background invalid'}='The default background for the specified theme is invalid';
  $strings{'Exclusive background'}='The specified background is not valid for this theme';
  $strings{'Bad background'}='The specified background is invalid';
  
  Abort("Bad theme error: $thErr.") if($thErr && index($thErr,join '@',keys %strings)!=-1);
  Abort("Bad bg error: $bgErr.") if($bgErr && index($bgErr,join '@',keys %strings)!=-1);
  if($thErr && $bgErr) {
    my $temp=$strings{$bgErr};
    substr($temp,0,1)=lc(substr($temp,0,1)); # downcase first letter
    return $strings{$thErr}." and $temp, so instead we're using the $thUse theme and the $bgUse background.";
  } elsif($thErr) {
    return $strings{$thErr}.", so we're using the $thUse theme instead.";
  } elsif($bgErr) {
    return $strings{$bgErr}.", so we're using the $bgUse background instead.";
  }
};


$L='fr'; ### French ###

$Matrix{'Navigation'}{$L}='Navigation';
$Matrix{'Themes'}{$L}='Th&egrave;mes';
$Matrix{'Theme'}{$L}='Th&egrave;me';
$Matrix{'Background'}{$L}='Fond';
$Matrix{'Change'}{$L}='Changer';
$Matrix{'Random Quote'}{$L}='Citation Al&eacute;atoire';
$Matrix{'Welcome to'}{$L}='Bienvenue &agrave;';
$Matrix{'Counter'}{$L}='Vous &ecirc;tes le visiteur N&ordm; ';

$Matrix{'Weblog Index'}{$L}='Index de Weblog';
$Matrix{'Weblog Navigation'}{$L}='Navigation de Weblog';
$Matrix{'Weblog entry'}{$L}='&Eacute;criture weblog N&ordm; ';
$Matrix{'Search'}{$L}='Rechercher';
$Matrix{'Untitled Page'}{$L}='Page Sans Titre';
$Matrix{'Search Results'}{$L}='R&eacute;sultats de la Recherche';
$Matrix{'plus'}{$L}='plus';
$Matrix{'minus'}{$L}='sans';
### $Matrix{'Lockfile stuck'}{$L}='A lockfile is stuck; please send me e-mail to let me know.';
### $Matrix{'Language ignored'}{$L}='The specified language contains invalid characters and will be ignored.';
$Matches{$L}=sub {
  my($num)=@_;
  return ('(Aucuns r&eacute;sultats)','(Un r&eacute;sultat)',"($num r&eacute;sultats)")[$num<2?$num:2];
};


$L='it'; ### Italian ###

$Matrix{'Navigation'}{$L}='Navigazione';
$Matrix{'Themes'}{$L}='Temi';
$Matrix{'Theme'}{$L}='Tema';
$Matrix{'Background'}{$L}='Background';
$Matrix{'Change'}{$L}='Cambiare';
$Matrix{'Random Quote'}{$L}='Citazione Casuale';
$Matrix{'Welcome to'}{$L}='Benvenuto a';
$Matrix{'Counter'}{$L}='Sei il visitatore N&ordm; ';

$Matrix{'Weblog Index'}{$L}='Indice del Weblog';
$Matrix{'Weblog Navigation'}{$L}='Navigazione del Weblog';
$Matrix{'Weblog entry'}{$L}='Entrata di weblog N&ordm; ';
$Matrix{'Search'}{$L}='Cerca';
$Matrix{'Untitled Page'}{$L}='Pagina Senza Titolo';
$Matrix{'Search Results'}{$L}='Risultati Ricerca';
$Matrix{'plus'}{$L}='pl&ugrave;';
$Matrix{'minus'}{$L}='meno';
### $Matrix{'Lockfile stuck'}{$L}='A lockfile is stuck; please send me e-mail to let me know.';
### $Matrix{'Language ignored'}{$L}='The specified language contains invalid characters and will be ignored.';
$Matches{$L}=sub { # nessun risultati ,  ci non sono risultati ...?
  my($num)=@_;
  return ('(Nessun risultati)','(Un risultato)',"($num risultati)")[$num<2?$num:2];
};


$L='pt'; ### Portuguese ###

$Matrix{'Navigation'}{$L}='Navega&ccedil;&atilde;o';
$Matrix{'Themes'}{$L}='Temas';
$Matrix{'Theme'}{$L}='Tema';
$Matrix{'Background'}{$L}='Fundo';
$Matrix{'Change'}{$L}='Mudar';
$Matrix{'Random Quote'}{$L}='Cita&ccedil;&otilde;es Aleat&oacute;rias';
$Matrix{'Welcome to'}{$L}='Boa Vinda a';
$Matrix{'Counter'}{$L}='Voc  o visitante N&ordm; ';

$Matrix{'Weblog Index'}{$L}='&Iacute;ndice do Weblog';
$Matrix{'Weblog Navigation'}{$L}='Navega&ccedil;&atilde;o do Weblog';
$Matrix{'Weblog entry'}{$L}='Entrada de Weblog N&ordm; ';
$Matrix{'Search'}{$L}='Buscar';
$Matrix{'Untitled Page'}{$L}='Page Sans Titre';
$Matrix{'Search Results'}{$L}='Buscar Resultados';
$Matrix{'plus'}{$L}='mais';
$Matrix{'minus'}{$L}='menos';
### $Matrix{'Lockfile stuck'}{$L}='A lockfile is stuck; please send me e-mail to let me know.';
### $Matrix{'Language ignored'}{$L}='The specified language contains invalid characters and will be ignored.';


$L='ja'; ### Japanese ###

$Matrix{'Welcome to'}{$L}='&#x3088;&#x3046;&#x3053;&#x305D;&#x3078;';


$L='xx-piglatin'; ### Pig Latin ###

$Matrix{'Navigation'}{$L}='Avigationnay';
$Matrix{'Themes'}{$L}='Emesthay';
$Matrix{'Theme'}{$L}='Emethay';
$Matrix{'Background'}{$L}='Ackgroundbay';
$Matrix{'Change'}{$L}='Angechay';
$Matrix{'Random Quote'}{$L}='Andomray Otequay';
$Matrix{'Welcome to'}{$L}='Elcomeway otay';
$Matrix{'Counter'}{$L}='Ouyay areway isitorvay #';

$Matrix{'Weblog Index'}{$L}='Eblogway Indexway';
$Matrix{'Weblog Navigation'}{$L}='Eblogway Avigationnay';
$Matrix{'Weblog entry'}{$L}='Eblogway entryway #';
$Matrix{'Search'}{$L}='Earchsay';
$Matrix{'Untitled Page'}{$L}='Untitledway Agepay';
$Matrix{'Search Results'}{$L}='Earchsay Esultsray';
$Matrix{'plus'}{$L}='usplay';
$Matrix{'minus'}{$L}='inusmay';
$Matrix{'Lockfile stuck'}{$L}='Away ockfilelay isway uckstay; easeplay endsay emay eway-ailmay otay etlay emay owknay.';
$Matrix{'Language ignored'}{$L}='Ethay ecifiedspay anguagelay ontainscay invalidway aracterschay andway illway ebay ignoredway.';
$Matches{$L}=sub {
  my($num)=@_;
  return ('(Onay atchesmay)','(1 atchmay)',"($num atchesmay)")[$num<2?$num:2];
};


$L='xx-hacker'; ### l33t-speak ###

$Matrix{'Navigation'}{$L}='n4v1g4+10n';
$Matrix{'Themes'}{$L}='+h3m3z';
$Matrix{'Theme'}{$L}='+h3m3';
$Matrix{'Background'}{$L}='b4x0rgr0und';
$Matrix{'Change'}{$L}='ch4ng3';
$Matrix{'Random Quote'}{$L}='r4nd0m qu0+3';
$Matrix{'Welcome to'}{$L}='\/\/3lc0m3 +0';
$Matrix{'Counter'}{$L}='j00 4r3 v151+0r #';

$Matrix{'Weblog Index'}{$L}='w3bl0g 1nd3x';
$Matrix{'Weblog Navigation'}{$L}='w3bl0g n4v1g4+10n';
$Matrix{'Weblog entry'}{$L}='w3bl0g 3n+ry #';
$Matrix{'Search'}{$L}='534r(h';
$Matrix{'Untitled Page'}{$L}='unt1tl3d p4g3';
$Matrix{'Search Results'}{$L}='534r(h r35ul+z';
$Matrix{'plus'}{$L}='plu5';
$Matrix{'minus'}{$L}='m1nu5';
$Matrix{'Lockfile stuck'}{$L}='4 l0ckf1l3 1z 5+ux0rz; plz 53nd m3 3-m41l +0 l3+ m3 kn0w, kthx.';
$Matrix{'Language ignored'}{$L}='+h3 5p3c1f13d l4ngu4g3 (0n+41nz 1nv4l1d (h4r4c+3rz 4nd w1ll b3 1gn0r3d.';
$Matches{$L}=sub {
  my($num)=@_;
  return ('(n0 m4+ch0rz)','(0n3 m4tch0r)',"($num m4+ch0rz)")[$num<2?$num:2];
};


$L='da'; ### Danish ###

$Matrix{'Welcome to'}{$L}='Velkommen hen til';


$L='nl'; ### Dutch ###

$Matrix{'Welcome to'}{$L}='Welkomstgroet voor';



######################################################################

sub ChooseLanguage {
  # Arguments: list of available languages (case-insensitive)
  # Returns: best available language

  my @avail=@_;
  _GetAcceptableLanguages() unless($GotLanguages);
  
  my($pri,$sub,$q)=('en','',.001); # default, even if not in @avail
  for(my $i=0;$i<@Primary;$i++) {
    foreach my $avail (@avail) {
      if($Quality[$i]>$q || ($Quality[$i]==$q && !$sub)) {
        if($avail=~/^(.*)-(.*)$/) {
          if($Primary[$i] eq $1 && $Secondary[$i] eq $2) {
            # both primary and sub match
            ($pri,$sub,$q)=($Primary[$i],$Secondary[$i],$Quality[$i]);
          }
        } else {
          if($Primary[$i] eq $avail && $avail ne 'xx') {
            # primary matches
            ($pri,$sub,$q)=($Primary[$i],'',$Quality[$i]);
          }
        }
      }
    }
  }
  my $lang=$sub?"$pri-$sub":$pri;
  $Languages{$lang}=1;
  return $lang;
}

sub Translate {
  # Arguments:  token
  # Returns:  translated string, or token if unable to translate

  my($token)=@_;

  my($lang)=ChooseLanguage(keys %{$Matrix{$token}});
  return $Matrix{$token}{$lang}?$Matrix{$token}{$lang}:$token;
}

sub ShortDate {
  # Arguments:  numeric date
  # Returns:  localized short date string
  
  my($num)=@_;
  
  my %ShortDates=(
    'en'    => 'm/d/yy',
    'en-gb' => 'dd/mm/yy',
    'es'    => 'dd/mm/yy',
    'es-us' => 'm/d/yy',
    'de'    => 'dd.mm.yy',
    'fr'    => 'dd/mm/yy',
    'fr-ca' => 'yy-mm-dd',
    'fr-ch' => 'dd.mm.yy',
    'it'    => 'dd/mm/yy',
    'it-ch' => 'dd.mm.yy',
    'pt'    => 'dd-mm-yy',
    'pt-br' => 'dd/mm/yy',
    'ja'    => '&#x5e73;&#x6210; heisei/mm/dd',
  );
    
  my($d,$m,$y,$w)=(localtime($num))[3,4,5,6];
  $m+=1;
  $y+=1900;
  my $dd=sprintf('%02u',$d);
  my $mm=sprintf('%02u',$m);
  my $yy=substr($y,2);
  my $heisei=$y-1988;
  
  my $short=($num && $num>0)?$ShortDates{ChooseLanguage(keys %ShortDates)}:'';
  $short=~s/mm/$mm/;
  $short=~s/m/$m/;
  $short=~s/dd/$dd/;
  $short=~s/d/$d/;
  $short=~s/yyyy/$y/;
  $short=~s/yy/$yy/;
  $short=~s/heisei/$heisei/;

  return $short;
}

sub LongDate {
  # Arguments:  numeric date
  # Returns:  localized long date string
  
  my($num)=@_;
  return '' if(!$num || $num=~/\D/);
  
  my %LongDates=(
    'en'    => '%Week, %Month %d<sup>%ord</sup>, %yyyy',
    'en-gb' => '%Week, %d<sup>%ord</sup> %Month %yyyy',
    'es'    => '%Week %d de %Month de %yyyy',
    'de'    => '%Week, %d. %Month %yyyy',
    'fr'    => '%Week %d %Month %yyyy',
    'fr-ch' => '%Week, %d %Month %yyyy',
    'it'    => '%Week %d %Month %yyyy',
    'pt'    => '%Week, %d de %Month de %yyyy',
    'ja'    => '&#x5e73;&#x6210;%heisei&#x5e74;%m&#x6708;%d&#x65e5;%Week&#x66dc;&#x65e5;',
    'xx-piglatin' => '%Week, %Month %d<sup>%ord</sup>, %yyyy',
    'xx-hacker'   => '%Week, %Month %d<sup>%ord</sup>, %yyyy',
  );
  my %WeekNames=(
    'en' => [ qw(Sunday Monday Tuesday Wednesday Thursday Friday Saturday) ],
    'es' => [ qw(Domingo Lunes Martes Mi&eacute;rcoles Jueves Viernes S&aacute;bado) ],
    'de' => [ qw(Sonntag Montag Dienstag Mittwoch Donnerstag Freitag Samstag) ],
    'fr' => [ qw(Dimanche Lundi Mardi Mercredi Jeudi Vendredi Samedi) ],
    'it' => [ qw(Domenica Luned&igrave; Marted&igrave; Mercoled&igrave; Gioved&igrave; Venerd&igrave; Sabato) ],
    'pt' => [ qw(Domingo Segunda-feira Ter&ccedil;a-feira Quarta-feira Quinta-feira Sexta-feira S&aacute;bado) ],
    'ja' => [ '&#x65e5;','&#x6708;','&#x706b;','&#x6c34;','&#x6728;','&#x91d1;','&#x571f;' ],
    'xx-piglatin' => [ qw(Undaysay Ondaymay Uesdaytay Ednesdayway Ursdaythay Idayfray Aturdaysay) ],
    'xx-hacker' => [ qw(5und4y m0nd4y +u35d4y w3dn3zd4y +hur5d4y fr1d4y 54turd4y) ],
  );
  my %MonthNames=(
    'en' => [ qw(January February March April May June July August September October November December) ],
    'es' => [ qw(enero febrero marzo abril mayo junio julio agosto septiembre octubre noviembre diciembre) ],
    'de' => [ qw(Januar Febuar M&auml;rz April Mai Juni Juli August September Oktober November Dezember) ],
    'fr' => [ qw(janvier f&eacute;vrier mars avril mai juin juillet aout septembre octobre novembre decembre) ],
    'it' => [ qw(gennaio febbraio marzo aprile maggio giugno luglio agosto settembre ottobre novembre dicembre) ],
    'pt' => [ qw(janeiro fevereiro mar&ccedil;o abril maio junho julho agosto setembro outubro novembro dezembro) ],
    'xx-piglatin' => [ qw(Anuaryjay Ebruaryfay Archmay Aprilway Aymay Unejay Ulyjay Augustway Eptembersay Octoberway Ovembernay Ecemberday) ],
    'xx-hacker' => [ qw(j4nu4ry f3bru4ry m4r\(h 4pr1l m4y jun3 july 4ugu5+ 53pt3mb3r 0c+0b3r n0v3mb3r d3\(3mb3r) ],
  );

  my($d,$m,$y,$w)=(localtime($num))[3,4,5,6];
  $m+=1;
  $y+=1900;
  my $dd=sprintf('%02u',$d);
  my $mm=sprintf('%02u',$m);
  my $yy=substr($y,2);
  my $heisei=$y-1988;
  my $ord=(qw(th st nd rd))[(($d<11 || $d>13) && substr($d,-1,1)<4)?substr($d,-1,1):0];

  my $language=ChooseLanguage(keys %LongDates);
  my $mainlang=substr($language,0,2);
  $mainlang=$language if($mainlang eq 'xx');
  
  my $Week=$WeekNames{$mainlang}[$w];
  my $Month=$MonthNames{$mainlang}[$m-1];
  
  my $long=$LongDates{$language};
  $long=~s/%Week/$Week/;
  $long=~s/%Month/$Month/;
  $long=~s/%mm/$mm/;
  $long=~s/%m/$m/;
  $long=~s/%dd/$dd/;
  $long=~s/%d/$d/;
  $long=~s/%yyyy/$y/;
  $long=~s/%yy/$yy/;
  $long=~s/%heisei/$heisei/;
  $long=~s/%ord/$ord/;

  return $long;
}

sub SearchMatches {
  # Arguments: number of matches
  # Returns: localized string for number of matches

  my($num)=@_;
  my($lang)=ChooseLanguage(keys %Matches);
  return &{$Matches{$lang}}($num);
}

sub GetLanguages {
  # Returns: string of languages used separated by semicolons
  return join('; ',sort keys %Languages);
}

sub ThemeError {
  # Arguments: theme error token, background error token, theme to use, background to use
  # Returns: localized alert string
  
  my($lang)=ChooseLanguage(keys %ThemeError);
  return &{$ThemeError{$lang}}(@_);
}

sub LocalizeFilename {
  # Arguments: filename without language, list of available filenames
  # Extension is optional, but do not include path
  # Returns: filename with language, or "" if not found

  my($filename,@files)=@_;
  my %languages=();
  my $extension=($filename=~s/(\.[^\.]+)$// ? $1 : '');
  foreach my $localized (@files) {
    if($localized eq $filename.$extension) {
      $languages{'en'}=$localized; # Assume any file without a language is 'en'
    } elsif($localized=~/^${filename}_(.*)${extension}$/) {
      $languages{$1}=$localized;
    }
  }
  if(%languages) {
    return $languages{ChooseLanguage(keys %languages)};
  } else {
    return ''; # not found in list
  }
}

sub GenerateCaptcha {
  # This is a pretty lazy way of doing this, but hopefully
  # spammers won't take the time to work around it, even though
  # to do so would be quite simple.
  
  # We will pick two numbers to either add or subtract.
  # The answer, as a string, will be MD5 hashed and passed
  # in a hidden field.  First we pick the question and answer:
  my($a,$b,$result,$op);
  if(rand(10)>5) { # ADDITION
    $a=int(rand(11)); # 0-10
    $b=int(rand(11)); # 0-10
    $result=$a+$b;
    $op=Translate('plus');
  } else { # SUBTRACTION
    $a=int(rand(11))+5; # 5-15
    $b=int(rand(6));    # 0-5
    $result=$a-$b;
    $op=Translate('minus');
  }
  # Next we generate the MD5 hash, and return the HTML:
  my @chars=('a'..'z', 'A'..'Z', '0'..'9');
  my $salt='$1$';
  $salt.=$chars[int(rand(@chars))] for(1..8);
  my $hash=substr(crypt(NumToString($result),$salt),3); # strip telltale $1$ from beginning of string
  return NumToString($a).qq/ $op <input type="hidden" name="captchakey" value="$hash"> /.NumToString($b);
}

sub NumToString {
  # Arguments:  integer, such as "3"
  # Returns:  string, such as "three" (must be all lower-case)

  my($int)=@_;
  my %numbers;
  $numbers{'en'}=[qw/zero one two three four five six seven eight nine ten
      eleven twelve thirteen fourteen fifteen sixteen seventeen eighteen
      nineteen twenty/];
  $numbers{'es'}=[qw/cero uno dos tres cuatro cinco seis siete ocho nueve
      diez once doce trece catorce quince diecis&eacute;is diecisiete 
      dieciocho diecinueve veinte/];
  $numbers{'de'}=[qw/null eins zwei drei vier f&uuml;nf sechs sieben acht
      neun zehn elf zw&ouml;lf dreizehn vierzehn f&uuml;nfzehn sechzehn
      siebzehn achtzehn neunzehn zwanzig/];
  $numbers{'fr'}=[qw/z&eacute;ro un deux trois quatre cinq six sept huit
      neuf dix onze douze treize quatorze quinze seize dix-sept dix-huit
      dix-neuf vingt/];
  $numbers{'pt'}=[qw/zero um dois tr&ecirc;s quatro cinco seis sete
      oito nove dez onze doze treze catorze quinze dezasseis dezasseis
      dezassete dezoito dezanove vinte/];
  $numbers{'xx-piglatin'}=[qw/erozay oneway ootay reethay ourfay ivefay ixsay
      evensay eightway inenay entay elevenway elvetway irteenthay ourteenfay
      ifteenfay ixteensay eventeensay eighteenway ineteennay wentytay/];
  $numbers{'xx-hacker'}=[qw/z3r0 0n3 tw0 thr33 f0ur f1v3 s1x s3v3n 31ght n1n3
      t3n 3l3v3n tw3lv3 th1rt33n f0urt33n f1ft33n s1xt33n s3v3nt33n 31ght33n
      n1n3t33n tw3nty/];

  my($lang)=ChooseLanguage(keys %numbers);
  return $numbers{$lang}->[$int];
}

sub _GetAcceptableLanguages {
  # Returns:  a list of the user's preferred languages
  
  # Should only be called if $GotLanguages is false

  my $error=undef;
  $GotLanguages=1;
  my $accept=(exists $ENV{'HTTP_ACCEPT_LANGUAGE'})?$ENV{'HTTP_ACCEPT_LANGUAGE'}:'';
  $accept=~s/\s//g;
  if(exists $Browser::Query{'language'}) {
    if($Browser::Query{'language'}=~/[^a-zA-Z\-]/) {
      $error='Language ignored';
    } else {
      $accept.=($accept?",":"").$Browser::Query{'language'}.';q=2';
    }
  }

  (@Primary,@Secondary,@Quality)=();
  foreach my $lang (split /,/,$accept) {
    my($pri,$sub,$q)=(lc($lang),'',1);
    if($pri=~/^(.*);q=(.*)$/) {
      $pri=$1;
      $q=$2;
    }
    if($pri=~/^(.*)-(.*)$/) {
      $pri=$1;
      $sub=$2;
    }
    # treat these as synonyms for en-gb
    $sub='gb' if($pri eq 'en' && $sub && index('au nz ie za ca',$sub)>-1);
    
    push(@Primary,$pri);
    push(@Secondary,$sub);
    push(@Quality,$q);
  }
  Alert(Translate($error)) if($error);
}



1;
