package Page;
#
# phroggy.com 5.4
#
# (C) 2009 Andy Lyttle, all rights reserved.
# This code may be distributed and modified under the terms of
# the GNU General Public License.
# http://phroggy.com/opensource/
# 

use strict;
use warnings;
use Paths;
use Counter;
use Quote;
use PhroggyIO;
use PhroggyHTML;
use Box;
use Blog;
use Localize;
use Navigation;
use Dynamic;

sub new {
  # Arguments:  hash of values (Filename, Title, Style, LinkUp)
  my($pkg,%values)=@_;
  
  # default values will be overridden by arguments
  my $attr=bless {('Title'=>Translate('Untitled Page'),'Style'=>'page',%values)}, $pkg;
  return $attr;
}

sub load {
  # Arguments: filename (without path or .page extension)
  
  my($pkg,$filename)=@_;
  my $docpath=$Paths::File{'docs'};

  my %pageinfo=('Filename'=>$filename,'Title'=>Translate('Untitled Page'),'Style'=>'page');
  my @boxlist=();
  
  if($filename=~/^weblog-(\d+)$/) {
    # blog
    return LoadBlog($1);
  } else {
    # not blog
    my $data=ReadFile("$docpath/$filename.page",'nocache');
    unless($data) {
      $filename='filenotfound';
      $data=ReadFile("$docpath/$filename.page",'nocache');
      Abort("Unable to read document files.") unless($data);
    }
    my %info=();
    foreach my $line (split(/\n/,$data)) {
      if($line=~/^LAYOUTSTYLE:(.*)$/) {
        $pageinfo{'Style'}=$1;
      } elsif($line=~/^TITLE:(.*)$/) {
        $pageinfo{'Title'}=$1;
      } elsif($line=~/^PATH:(.*)$/) {
        $pageinfo{'Path'}=$1;
      } elsif($line=~/^SHORTTITLE:(.*)$/) {
        $pageinfo{'ShortTitle'}=$1;
      } elsif($line=~/^DESCRIPTION:(.*)$/) {
        $pageinfo{'Description'}=$1;
      } elsif($line=~/^LINKUP:(.*)$/) {
        $pageinfo{'LinkUp'}=$1;
      } elsif($line=~/^ICON:(.*)$/) {
        $pageinfo{'Icon'}=$1;
      } elsif($line=~/^ONLOAD:(.*)$/) {
        $pageinfo{'OnLoad'}=$1;
      } elsif($line=~/^HEADTAG:(.*)$/) {
        push @{$pageinfo{'HeadTags'}},$1;
      } elsif($line=~/^LOADBOX:(.*)$/) {
        push(@boxlist,new DocBox(%info)) if(exists $info{'Title'});
        %info=();
        my $file=$1;
        if($file=~/[^a-zA-Z0-9\-\.]/) {
          push(@boxlist,error DocBox("<p>LOADBOX line in $filename.page contains an invalid character.</p>"));
        } else {
          push(@boxlist,load DocBox($file));
        }
      } elsif($line=~/^DYNAMIC:(.*)$/) {
        push(@boxlist,new DocBox(%info)) if(exists $info{'Title'});
        %info=();
        if($1 eq 'blogindex') {
          push(@boxlist,GetBlogIndex());
        } elsif($1 eq 'recentblog') {
          push(@boxlist,GetRecentBlog());
        } elsif($1 eq 'welcome') {
          push(@boxlist,GetWelcomeBox());
        } else {
          push(@boxlist,error DocBox("<p>DYNAMIC line in $filename.page refers to nonexistent functionality.</p>"));
        }
      } elsif($line=~/^WINDOW:(.*)$/) {
        push(@boxlist,new DocBox(%info)) if(exists $info{'Title'});
        %info=('Title'=>$1);
      } elsif($line=~/^SUBTITLE:(.*)$/) {
        $info{'Subtitle'}=$1;
      } elsif($line=~/^DATE:(.*)$/) {
        $info{'Date'}=DateToNum($1); # will set to 0 if not valid format
      } elsif($line=~/^CLASS:(.*)$/) {
        $info{'Class'}=$1;
      } elsif($line=~/^BOXID:(.*)$/) {
        $info{'BoxID'}=$1;
      } elsif($line ne '.') {
        # ignore lines containing only . for legacy support
        $info{'Body'}.=$line."\n";
      }
    }
    push(@boxlist,new DocBox(%info)) if(exists $info{'Title'});
  
 # Increment counter and log hit
 # FIXME - move this elsewhere, and call it from each script
 SetLock();
 GetCounter();
 LogHit($filename);
 ClearLock();  
    
    $pageinfo{'DocList'}=\@boxlist;
    my $attr=bless \%pageinfo;
    return $attr;
  }
}

sub AddBox {
  # Arguments:  list of doc windows
  
  my($self,@doclist)=@_;
  push @{$self->{'DocList'}},@doclist;
}

sub Set {
  # Arguments:  hash of values (Filename, Title, Style, LinkUp)
  
  my($self,%values)=@_;
  %$self=(%$self,%values);  
}

sub AddHead {
  # Arguments:  head tag
  
  my($self,$tag)=@_;
  push @{$self->{'HeadTags'}},$tag;
}

sub Print {
  # Arguments:  force new quote? (boolean)

  # Print HTTP headers and the HTML page
  
  my($self,$newquote)=@_;
  my $docpath=$Paths::File{'docs'};
  my $quote=new Quote($newquote);
  
  # Read layout .tmpl file
  my $html='';
  if($self->{'Style'} eq 'home') {
    $html=$main::Theme->GetLayoutTemplate('Home.tmpl');
  } else {
    $html=$main::Theme->GetLayoutTemplate('Page.tmpl');
  }
  $html=~s/{HEADTAGS}/_HeadTags($self)/eg;
  $html=~s/{CONTENT}/_RenderBoxes(GetAlerts(),@{$self->{'DocList'}})/eg;
  $html=~s!{LOADFILE:(.*?)}!ReadFile("$docpath/$1")!eg;
  $html=~s!{LOADBOX:(.*?)}!_RenderBoxes(load DocBox($1))!eg;
  $html=~s!{LOADMENU:(.*?)}!_RenderBoxes(load MenuBox($1))!eg;
  $html=~s!{DYNAMICMENU:(.*?)}!_RenderBoxes(GetDynamicMenu($1,$self->{'Filename'}))!eg;
  $html=~s!{SPAMTRAP}!GetSpamTrap()!eg;
  $html=~s!{ONLOAD}!$self->{'OnLoad'}?' onLoad="'.$self->{'OnLoad'}.'"':''!eg;
  # Finished reading layout .tmpl file
  
  # Variable substitution
  $html=~s/{BASEPATH}/$Paths::Web{'base'}/g;
  $html=~s/{CGIPATH}/$Paths::Web{'cgi'}/g;
  $html=~s!{THEMEPATH}!$Paths::Web{'themes'}.'/'.$main::Theme->Name!eg;
  $html=~s!{LAYOUTPATH}!$Paths::Web{'layouts'}.'/'.$main::Theme->Layout!eg;
  $html=~s/{FILENAME}/$self->{'Filename'}/g;
  $html=~s/{LINKTO:(.*?)}/LinkTo($1)/eg;
  $html=~s/{GOTHEME:(.*?)}/LinkTo($self->{'Filename'},$1)/eg;
  $html=~s/{URI}/GetURI($self->{'Filename'})/eg;
  $html=~s/{URI:(.*?)}/GetURI($1)/eg;
  $html=~s/{QUOTE}/$quote->Quote/eg;
  $html=~s/{COUNT}/$main::Count/g;
  $html=~s/{CAPTCHA}/GenerateCaptcha()/eg;
  $html=~s/{SEARCH_TERMS}/defined($Browser::Query{'q'})?Escape($Browser::Query{'q'}):''/eg;
  
  # Print HTTP headers (see RFC 2616 section 14)

  # Set cookies
  my $exp=_GetExpiration();
  print 'Set-cookie: theme=',$main::Theme->Name,"; path=$Paths::Web{'base'}/; expires=$exp\n";
  print 'Set-cookie: background=',$main::Background->Name,"; path=$Paths::Web{'base'}/; expires=$exp\n";
  print 'Set-cookie: viewedquotes=',$quote->CookieString,"; path=$Paths::Web{'base'}/; expires=$exp\n";
  print 'Set-cookie: quote=',$quote->Number,"; path=$Paths::Web{'base'}/\n"; # expires when browser exits
  # list all languages used in generating this page, in no particular order
  print 'Content-language: ',GetLanguages(),"\n";
  # instruct caching proxy servers that we're serving dynamic content which can change based on things like cookies
  print "Vary: *\n";
  # content rating for the entire site, see http://www.icra.org/ and http://www.w3.org/PICS/
  print 'PICS-Label: (pics-1.1 "http://www.icra.org/ratingsv02.html" l gen true for "';
  print "$Paths::Web{'root'}$Paths::Web{'base'}/\" r (nz 1 vz 1 lz 1 oz 1 cz 1))\n";
  # content type, character set, and length
  print "Content-type: text/html; charset=utf-8\n";
  print 'Content-length: ',length($html),"\n\n";
  print $html;
}

sub _GetExpiration {
  # Returns: date string, formatted like Thursday, 01-Jan-04 0:00:00 GMT
  # See RFC 2616 section 3.3.1, RFC 850 section 2.1.4

  my $daysahead=180; # when to expire
  my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday)=gmtime(time+(86400*$daysahead));
  return sprintf '%s, %02u-%s-%02u %u:%02u:%02u GMT',
    (qw(Sunday Monday Tuesday Wednesday Thursday Friday Saturday))[$wday],
    $mday+1,
    (qw(Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec))[$mon],
    $year+1900,
    $hour,$min,$sec;
}

sub _RenderBoxes {
  # Arguments: array of Box objects
  # Returns: string of HTML generated by Render() of each Box
  
  my $html='';
  $html.=$_->Render."\n" foreach (@_);
  chomp($html);
  return($html);
}

sub _HeadTags {
  # Arguments:  Properties
  # Returns:  HTML for link tags, description, body CSS and spamtrap to go in HEAD section
  
  my($props)=@_;
  
  my $html="\t<!-- Generated by phroggy.com 5.4 -->\n";
  $html.="\t<!-- Filename: $props->{'Filename'} -->\n" if($props->{'Filename'});
  $html.="\t<!-- Theme: ".$main::Theme->Title." (".$main::Theme->Name.") -->\n";
  $html.="\t<!-- Background: ".$main::Background->Title." (".$main::Background->Name.") -->\n";
  $html.="\t<!-- Layout: ".$main::Theme->Layout." -->\n";
  $html.="\t<!-- Languages: ".GetLanguages()." -->\n";

  $html.="\t<title>".$props->{'Title'}."</title>\n";
  $html.="\t".'<meta name="description" content="'.$props->{'Description'}.'">'."\n" if($props->{'Description'});
  $html.="\t".'<link rel="stylesheet" href="'.$Paths::Web{'layouts'}.'/'.$main::Theme->Layout.'/stylesheet.css" type="text/css">'."\n";
  $html.="\t".'<link rel="stylesheet" href="'.$Paths::Web{'bg'}.'/'.$main::Background->Name.'.css" type="text/css">'."\n";
  $html.="\t".'<link rel="stylesheet" href="'.$Paths::Web{'themes'}.'/'.$main::Theme->Name.'/stylesheet.css" type="text/css">'."\n";
  $html.="\t".'<link rel="shortcut icon" href="'.$Paths::Web{'base'}.'/favicon.ico" type="image/x-icon">'."\n";
  $html.="\t".'<link rel="top" href="'.GetURI('home').'" title="'.GetDescription('home').'">'."\n";
  $html.="\t".'<link rel="up" href="'.GetURI($props->{'LinkUp'}).'" title="'.GetDescription($props->{'LinkUp'}).'">'."\n" if($props->{'LinkUp'});
  $html.="\t".'<link rel="first" href="'.GetURI($props->{'LinkFirst'}).'" title="'.GetDescription($props->{'LinkFirst'}).'">'."\n" if($props->{'LinkFirst'});
  $html.="\t".'<link rel="prev" href="'.GetURI($props->{'LinkPrev'}).'" title="'.GetDescription($props->{'LinkPrev'}).'">'."\n" if($props->{'LinkPrev'});
  $html.="\t".'<link rel="next" href="'.GetURI($props->{'LinkNext'}).'" title="'.GetDescription($props->{'LinkNext'}).'">'."\n" if($props->{'LinkNext'});
  $html.="\t".'<link rel="last" href="'.GetURI($props->{'LinkLast'}).'" title="'.GetDescription($props->{'LinkLast'}).'">'."\n" if($props->{'LinkLast'});
  $html.="\t$_\n" foreach @{$props->{'HeadTags'}};
  chomp $html;
  return($html);
}


1;
