package Quote;
#
# phroggy.com 5.06
#
# (C) 2004 Andy Lyttle, all rights reserved.
# This code may be distributed and modified under the terms of
# the GNU General Public License.
# http://phroggy.com/opensource/
# 

use strict;
use warnings;
use Paths;
use Browser;
use PhroggyIO;

# @ViewedQuotes = array indicating which quotes the user has already
#     seen, read by GetCookies(); each element in the array is 1 or 0
# @Q = array of quotes (text), read by ReadQuotes()
# $NumQuotes = length of @Q
# $quote = number of the quote that we've chosen to display


sub new {
  # Arguments:  force new quote?  (boolean)

  # Choose which quote should be displayed
  # If argument is true, ignore quote cookie
  
  my($pkg,$force)=@_;

  my($CookieNumQuotes,@ViewedQuotes)=_StringToBits($Browser::Cookies{'viewedquotes'});

  my @Q=_ReadQuotes();
  my $NumQuotes=scalar(@Q);
  my $quote=0;

  if(!$force && defined $Browser::Cookies{'quote'} &&
        $Browser::Cookies{'quote'}=~/^\d+$/ && $Browser::Cookies{'quote'}<$NumQuotes) {
    # User has already gotten a random quote; we'll continue to use it
    $quote=$Browser::Cookies{'quote'};
  } elsif($CookieNumQuotes>0 && $NumQuotes>$CookieNumQuotes) {
    # A new quote has been added to the list; we should pick that one
    $quote=$NumQuotes-1;
  } else {
    my @Unviewed=();
    for(my $x=0;$x<$NumQuotes;$x++) {
      # Add the number of each unviewed quote to a list
      push(@Unviewed,$x) if(!$ViewedQuotes[$x]);
    }
    if(@Unviewed) {
      # Choose the number of a quote from the list of unviewed numbers
      $quote=$Unviewed[int(rand(scalar(@Unviewed)))];
    } else {
      # There are no unviewed quotes; reset back to no viewed quotes
      # and pick one at random, or -1 if no quotes exist
      @ViewedQuotes=();
      $quote=($NumQuotes>0 ? int(rand($NumQuotes)) : -1);
    }
  }
  # Remember that the user has seen this one (or will have seen it once
  # this page is rendered)
  $ViewedQuotes[$quote]=1 if($quote>=0);
  
  return bless {'CookieString'=>_BitsToString($NumQuotes,@ViewedQuotes),
  'Quote'=>($quote>=0?$Q[$quote]:'No quotes available.'),'Number'=>$quote}, $pkg;
}

sub CookieString {
  my($self)=@_;
  return $self->{'CookieString'};
}

sub Quote {
  my($self)=@_;
  return $self->{'Quote'};
}

sub Number {
  my($self)=@_;
  return $self->{'Number'};
}


#####

sub _ReadQuotes {
  # Returns: list of quotes

  # Read all the random quotes from quotes.data in the webdocs directory
  
  my @Q=();
  if(open(QUOTES,"$Paths::File{'docs'}/quotes.data")) {
    my $x=-1;
    my $q='';
    while(<QUOTES>) {
      chomp;
      if($_ eq '') {
        if($q) {
          $x++;
          $Q[$x]=$q;
          $q='';
        }
      } else {
        $q.=$_."\n";
      }
    }
    if($q) {
      $x++;
      $Q[$x]=$q;
    }
    close(QUOTES);
  } else {
    Alert('Could not open list of random quotes.');
  }
  return @Q;
}

sub _CharToNum {
  # take one character, convert to number (0-63)
  my $ret=-1;
  my @Chars=('0'..'9', 'a'..'z', 'A'..'Z', '-', '_');
  foreach my $i (0..scalar(@Chars)) {
    if($_[0] eq $Chars[$i]) {
      $ret=$i;
      last;
    }
  }
  if($ret<0) {
    die("error, $_[0] not a valid char");
  }
  return($ret);
}

sub _NumToChar {
  # take a number (0-63), convert to one character
  return(('0'..'9', 'a'..'z', 'A'..'Z', '-', '_')[$_[0]]);
}

sub _BitsToNum {
  # take 6 bits, convert to number (0-63)
  my(@array)=@_;
  my $val=0;
  for(my $x=0;$x<6;$x++) {
    $array[$x]=0 unless($array[$x]);
    my $y=2**(5-$x);
    $val+=($y*$array[$x]);
  }
  return($val);
}

sub _NumToBits {
  # take a number (0-63), convert to array of 6 bits
  my $val=$_[0];
  my @bits=(0,0,0,0,0,0);
  for(my $x=0;$x<6;$x++) {
    my $y=2**(5-$x);
    if($val>=$y) {
      $bits[$x]=1;
      $val-=$y;
    }
  }
  return(@bits);
}

sub _StringToBits {
  # take a string from the cookie, convert to the number of quotes and a binary array
  my @string=(0,0);
  @string=split //,$_[0] if($_[0]);
  my $a=shift(@string);
  my $b=shift(@string);
  my $num=_CharToNum($a)*64+_CharToNum($b);
  our @list=();
  foreach my $i (@string) {
    push(@list,_NumToBits(_CharToNum($i)));
  }
  return($num,@list);
}

sub _BitsToString {
  # take the number of quotes and a binary array, return a cookie string
  my @Seen=@_;
  my $b=shift(@Seen); # first item is length, not part of array
  my $a=int($b/64);
  $b-=($a*64);
  my $string=_NumToChar($a)._NumToChar($b);
  
  for(my $byte=0;$byte<@Seen;$byte+=6) {
    my @bits=@Seen[$byte..($byte+5)];
    $string.=_NumToChar(_BitsToNum(@bits));
  }
  return($string);
}


1;
